pro uti_avgband,band=band,chstart=chstart,chend=chend, all_bands=all_bands
;yes
;=Task:UTI_AVGBAND --- To create new continuum visibilities by averaging channel data.
;#Type: utility
;+Use:
;      The commnad UTI_AVGBAND can be used to average the uv data in
;      one or more chunks and make a new continuum channel c1. The
;      following are some examples:
;      >uti_avgband,band='s3' --- Averaging chunk3 data
;      >uti_avgband,band=['s1','s2','s3','s4'] --- Averaging data from chunk1
;                                                  to chunk4.   
;      >uti_avgband,band='s1',chstart=32,chend=64 --- Averaging chunk1 data,
;                                                     but only from ch32 to ch64;%history:
;&history:
;------------------------------------------------------------------------
;      cykuo 19feb04 Adapting the header
;------------------------------------------------------------------------   

common global
common data_set

red = [0,1,1,0,0,1]
green = [0,1,0,1,0,1]
blue = [0,1,0,0,1,0]
; color 2 = red
;	3 = green
;	4 = blue
;	5 = yellow

;tvlct, 255*red, 255*green, 255*blue

;result=dat_list(/reset)

if keyword_set(all_bands) then begin
 band=uti_distinct(c.band[sp[psl].iband],nband,/many_repeat)
 band=band[where(band ne 'c1')]
endif
distinct_sb=uti_distinct(c.sb[bl[pbl].isb],nsbs,/many_repeat)
distinct_blcd=uti_distinct(c.blcd[bl[pbl].iblcd],nblcds,/many_repeat)
distinct_rec=uti_distinct(c.rec[bl[pbl].irec],nrecs,/many_repeat)

for ir = 0,nrecs-1 do begin

for is = 0,nsbs-1 do begin
  freq=0.
  nfreq=0.
  for ib = 0,nblcds-1 do begin
     result=dat_list(s_l,'"wt" gt "0" and "band" eq "c1" and "sb"   eq "' $
        +distinct_sb[is] + '" and "blcd" eq "' +distinct_blcd[ib]+ $
        '" and "rec" eq "'+distinct_rec[ir]+'"',/reset,/no_notify)
     if sp[psl[0]].fres ne n_elements(band)*82. then sp[psl].fres=(sp[psl].fres/abs(sp[psl].fres)) * n_elements(band)*82.
     nc=n_elements(pil)
     nint=in[pil].int
     nb=intarr(nc)
     for k = 0L,n_elements(band)-1 do begin
        recheck: result=dat_list(s_l,'"wt" gt "0" and "band" eq "' + band[k]+ '" and "sb"   eq "' $
            +distinct_sb[is]+ '" and "blcd" eq "' +distinct_blcd[ib]+ $
            '" and "rec" eq "'+distinct_rec[ir]+'"',/reset,/no_notify)
        if result eq 0 then begin
          print, 'No data in '+band[k]
          print, 'Please flag out this whole band first. Quit !!!'
          return
       endif
       n_ch=sp[psl].nch+1
       ibad=where(abs(ch[pcl+n_ch/2]) eq 0., count)
       if (count gt 0.) then begin
          int_bad=strcompress(string(uti_distinct(in[pil[ibad]].int,nint_bad,/many_repeat)),/remove_all)          
          print, 'Automatic flagging bad points of integrations in baseline '+ $
                 distinct_blcd[ib]+ ' band '+band[k]+':'  
          print,int_bad
          sp[psl[ibad]].wt=-abs(sp[psl[ibad]].wt)
          re.wts[prl[ibad]]=-abs(re.wts[prl[ibad]])
          goto, recheck
       endif

;        kk=where((in[pil[1:*]].int-in[pil].int) ge 2, count)
;        if count gt 0 then deficit=in[pil[kk]].int+1 else deficit=-1
        freq=freq+total(sp[psl].fsky)
        nfreq=nfreq+n_elements(psl)
        dat_get_rows,cmp,amp,pha,x,wts,first,npts,'channel',js,/list,/amp_pha
;        print,'The number of points is ',n_elements(first)
        skip1 = fix(  (1 - 82./104.)*npts/2.  )
        skip2 = fix(  (1 - 82./104.)*npts/2.  )
        if (  keyword_set(chstart)) then begin
           skip1 = make_array(n_elements(npts),/long,value=chstart) 
        endif
        if (  keyword_set(chend)) then begin
           skip2 =  make_array(n_elements(npts),/long)
           skip2 =  npts - chend 
        endif

;print,'start end ',skip1[0],npts[0] - skip2[0] 


;print,'skip channels ',skip

        if (k eq 0) then begin
           avgreal =  fltarr(nc)
           avgimag =  fltarr(nc)
        endif
        for j = 0L, nc-1 do begin
          i=where(nint(j) eq in[pil].int, count)
          if count ne 0 then begin
  avgreal[j] = avgreal[j] + total(  float (cmp[   first[i]+skip1[i] : first[i]+npts[i]-skip2[i]-1 ])  )/(npts[i]-skip2[i]-skip1[i] )
  avgimag[j] = avgimag[j] + total(  imaginary(cmp[   first[i]+skip1[i] : first[i]+npts[i]-skip2[i]-1 ])  )/(npts[i]-skip2[i]-skip1[i] )
              nb[j]=nb[j]+1
          endif
        endfor             

     endfor
     n_nb=where(nb eq 0,count)
     if count gt 0 then begin
        int_nb=strcompress(string(nint(where(nb eq 0))),/remove_all)
        command='"rec" eq "'+distinct_rec[ir]+'" and "sb" eq "'+distinct_sb[is]+ '" and "blcd" eq "' $
            +distinct_blcd[ib]+ '" and ("int" eq "'+int_nb[0]+'"'
        if count gt 1 then for i=1,count-1 do command=command+' or "int" eq "'+ int_nb[i]+'"'
        command=command+')'
        print,command
        result=dat_list(s_l,command,/reset,/no_notify)
        sp[psl].wt = -abs(sp[psl].wt)
        re.wts[prl] = -abs(re.wts[prl] )        
     endif
     avgreal = avgreal(where(nb ne 0))/nb(where(nb ne 0))
     avgimag = avgimag(where(nb ne 0))/nb(where(nb ne 0))
;     print,'The number of average points is ',n_elements(avgreal)
     result=dat_list(s_l,'"wt" gt "0" and "band" eq "c1" and "sb"   eq "' +distinct_sb[is] $
        + '" and "blcd" eq "' +distinct_blcd[ib]+'" and "rec" eq "'+distinct_rec[ir]+'"',/reset,/no_notify)
;     print,'The number of continuum points is ',n_elements(pcl),n_elements(pbl)
     ch[pcl]=complex(avgreal,avgimag)

     phase2 = atan( avgimag,avgreal ) * 180./!pi
     amp2 = sqrt(avgimag*avgimag + avgreal*avgreal) 
     bl(pbl).ampave = amp2
     bl(pbl).phaave = phase2
     print,'Bands averaging done for baseline '+distinct_blcd[ib]+' '+distinct_sb[is]+ 'sb'+ ' '+'rx '+distinct_rec[ir]
     print,''
  endfor
  freq=freq/nfreq
  result=dat_list(s_l,'"band" eq "c1" and "sb" eq "' +distinct_sb[is]+ '" and "rec" eq "'+distinct_rec[ir]+'"',/reset,/no_notify)
  sp[psl].fsky=freq
endfor ; for sb
endfor ; for rec

print,'Finished bands averaging',band
result=dat_list(s_l,/reset)
end


